#!/usr/bin/env python3
# -*- coding:utf-8 -*-

"""
The Official CKIP CoreNLP Toolkit.
"""

__author_name__ = 'Mu Yang'
__author_email__ = 'emfomy@gmail.com'
__copyright__ = '2018-2020 CKIP Lab'

__title__ = 'CKIPNLP'
__version__ = '1.0.0'
__description__ = 'CKIP CoreNLP'
__license__ = 'GPL-3.0'

__url__ = 'https://ckipnlp.readthedocs.io'
__download_url__ = __url__+'/tarball/'+__version__
