#!/usr/bin/env python3
# -*- coding:utf-8 -*-

"""
This module implements specialized container datatypes for CKIPNLP.
"""

__author__ = 'Mu Yang <http://muyang.pro>'
__copyright__ = '2018-2020 CKIP Lab'
__license__ = 'GPL-3.0'

from .text import *
from .seg import *
from .ner import *
from .parse import *
from .coref import *

from .util.wspos import *
from .util.parse_tree import *
